package madqba.awt;

import madqba.awt.chat.Mediator;

import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Scrollbar;

import java.util.Date;
import java.util.StringTokenizer;


/**
 * ChatPanel is an awt component which is used to display conversation's course.
 *
 * @author Jakub Madej
 */
public class ChatPanel extends Container {
    public final static int HEIGHT = 2300;
    public final static int SCRBARWIDTH = 16;
    public final static int SCRBARPOS = 1685;
    public final static int SCRBARVIS = 300;
    public final static int LINEHEIGHT = 14;
    public final static int INVISIBLEHEIGHT = 2000;
    public final static int LINE1X = 65;
    public final static int LINE2X = 130;
    public final static int TEXTX = 140;
    public final static int FONTSIZE = 12;
    public final static int NICKLENGTH = 9;
    public final static int TIMEXPOS = 3;
    public final static int NICKXPOS = 97;
    public final static int TEXTLENGTH = 350;
    public final static int SPACE = 5;
    public final static Color COLOR1 = new Color(255, 255, 150);
    public final static Color COLOR2 = new Color(255, 150, 255);
    public final static Color COLOR3 = new Color(255, 255, 230);
    public final static Color COLOR4 = new Color(10, 105, 112);
    private MyCanvas canvas;
    private Scrollbar scrbar;
    private Mediator med;

    /**
     * Creates a new ChatPanel object.
     *
     * @param med controlling mediator
     * @param width width of the component
     * @param height height of the component
     */
    public ChatPanel(Mediator med, int width, int height) {
        this.med = med;
        setLayout(null);
        setSize(width, height);
        scrbar = new Scrollbar(Scrollbar.VERTICAL, HEIGHT - height, SCRBARVIS,
                0, HEIGHT - height);
        scrbar.setBounds(width - SCRBARWIDTH, 0, SCRBARWIDTH, height);
        scrbar.setVisible(false);
        add(scrbar);
        canvas = new MyCanvas(width - SCRBARWIDTH, height);
        canvas.setBounds(0, 0, width - SCRBARWIDTH, height);
        canvas.setVisible(false);
        scrbar.addAdjustmentListener(canvas);
        add(canvas);
    }

    /**
     * Sets visibility.
     *
     * @param b if true, shows this component; otherwise, hides this component
     */
    public void setVisible(boolean b) {
        scrbar.setVisible(b);
        canvas.setVisible(b);
        super.setVisible(b);
        repaint();
    }

    /**
     * Appends a new message.
     *
     * @param txt text to be appended
     * @param nick nickname of person who speeches
     */
    public void append(String txt, String nick) {
        if (nick.equals(med.getNick()))
            append(txt, nick, COLOR1);
        else if (nick.equals("!!"))
            append(txt, nick, COLOR2);
        else
            append(txt, nick, COLOR3);
    }

    /**
     * Appends a new message with specified color.
     *
     * @param txt text to be appended
     * @param nick nickname of person who speeches
     * @param color color to be used
     */
    public void append(String txt, String nick, Color color) {
        Date date = new Date();
        Graphics g = canvas.getImage().getGraphics();

        g.drawImage(canvas.getImage(), 0, -LINEHEIGHT, this);
        g.setColor(COLOR4);
        g.fillRect(0, INVISIBLEHEIGHT - LINEHEIGHT,
            canvas.getImage().getWidth(this), LINEHEIGHT);
        g.setColor(COLOR3);
        g.drawLine(LINE1X, 0, LINE1X, INVISIBLEHEIGHT);
        g.drawLine(LINE2X, 0, LINE2X, INVISIBLEHEIGHT);

        g.setFont(new Font("Monospaced", Font.PLAIN, FONTSIZE));

        String secs = String.valueOf(date.getSeconds());

        if (secs.length() == 1)
            secs = "0" + secs;

        String mins = String.valueOf(date.getMinutes());

        if (mins.length() == 1)
            mins = "0" + mins;

        String hs = String.valueOf(date.getHours());

        if (hs.length() == 1)
            hs = " " + hs;

        while (nick.length() < NICKLENGTH) {
            nick += " ";

            if (nick.length() < NICKLENGTH)
                nick = " " + nick;
        }

        g.drawString(hs + ":" + mins + ":" + secs, TIMEXPOS,
            INVISIBLEHEIGHT - LINEHEIGHT);

        g.setFont(new Font("Default", Font.PLAIN, FONTSIZE));

        g.drawString(nick,
            NICKXPOS - (g.getFontMetrics().stringWidth(nick) / 2),
            INVISIBLEHEIGHT - LINEHEIGHT);

        StringTokenizer strtok = new StringTokenizer(txt);
        String line = "";
        String token = null;

        if (strtok.hasMoreTokens())
            token = strtok.nextToken();

        for (;;) {
            if ((token != null) &&
                    (g.getFontMetrics().stringWidth(token) > TEXTLENGTH)) {
                char[] tab = token.toCharArray();
                line = token = "";

                int i = 0;

                for (; g.getFontMetrics().stringWidth(line) <= TEXTLENGTH;
                        i++)
                    line += tab[i];

                for (; i < tab.length; i++)
                    token += tab[i];
            } else

                while ((token != null) &&
                        (g.getFontMetrics().stringWidth(line + " " + token) < TEXTLENGTH)) {
                    if (token.equals(":)") || token.equals(":-)") ||
                            token.equals(":^)")) {
                        g.drawImage(med.getSmiles().getSmile1(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals(";(") || token.equals(";-(") ||
                            token.equals(";^(")) {
                        g.drawImage(med.getSmiles().getSmile2(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals(":/") || token.equals(":\\") ||
                            token.equals(":|") || token.equals(":-/") ||
                            token.equals(":-\\") || token.equals(":-|") ||
                            token.equals(":^/") || token.equals(":^\\") ||
                            token.equals(":^|")) {
                        g.drawImage(med.getSmiles().getSmile3(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals(":D") || token.equals(":-D") ||
                            token.equals(":^D")) {
                        g.drawImage(med.getSmiles().getSmile4(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals(":(") || token.equals(":-(") ||
                            token.equals(":^(")) {
                        g.drawImage(med.getSmiles().getSmile5(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals("8)") || token.equals("8-)") ||
                            token.equals("8^)")) {
                        g.drawImage(med.getSmiles().getSmile6(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals(":o") || token.equals(":O") ||
                            token.equals(":0") || token.equals(":-o") ||
                            token.equals(":-O") || token.equals(":-0") ||
                            token.equals(":^o") || token.equals(":^O") ||
                            token.equals(":^0")) {
                        g.drawImage(med.getSmiles().getSmile7(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals(":p") || token.equals(":P") ||
                            token.equals(":-p") || token.equals(":-P") ||
                            token.equals(":^p") || token.equals(":^P")) {
                        g.drawImage(med.getSmiles().getSmile8(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals(":]") || token.equals(":-]") ||
                            token.equals(":^]")) {
                        g.drawImage(med.getSmiles().getSmile9(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals("=)")) {
                        g.drawImage(med.getSmiles().getSmile10(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals("??")) {
                        g.drawImage(med.getSmiles().getSmile11(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else if (token.equals("!!")) {
                        g.drawImage(med.getSmiles().getSmile12(),
                            TEXTX + g.getFontMetrics().stringWidth(line) +
                            SPACE,
                            INVISIBLEHEIGHT - LINEHEIGHT -
                            g.getFontMetrics().getHeight() + SPACE, this);
                        line += "       ";
                    } else {
                        if (line.equals(""))
                            line = token;
                        else
                            line = line + " " + token;
                    }

                    token = null;

                    if (strtok.hasMoreElements())
                        token = strtok.nextToken();
                }

            g.setColor(color);
            g.drawString(line, TEXTX, INVISIBLEHEIGHT - LINEHEIGHT);

            if (token == null)
                break;

            g.drawImage(canvas.getImage(), 0, -LINEHEIGHT, this);
            g.setColor(COLOR4);
            g.fillRect(0, INVISIBLEHEIGHT - LINEHEIGHT,
                canvas.getImage().getWidth(this), LINEHEIGHT);
            g.setColor(COLOR3);
            g.drawLine(LINE1X, 0, LINE1X, INVISIBLEHEIGHT);
            g.drawLine(LINE2X, 0, LINE2X, INVISIBLEHEIGHT);

            line = "";
        }

        scrbar.setValue(SCRBARPOS);
        canvas.setScrbarPos(SCRBARPOS);
        canvas.repaint();
    }

    /**
     * Appends a new information message.
     *
     * @param txt text to be appended
     */
    public void append(String txt) {
        append(txt, "!!");
    }

    /**
     * Sets contents.
     *
     * @param txt text to be displayed
     */
    public void setText(String txt) {
        Graphics g = canvas.getImage().getGraphics();
        g.setColor(COLOR4);
        g.fillRect(0, 0, canvas.getImage().getWidth(this), INVISIBLEHEIGHT);
        append(txt);
    }
}
