package madqba.awt;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;


/**
 * This class implements a canvas, which contains an image. The image's height
 * is bigger the canvas'es size and the visible area is changed whenever
 * adjustmentValueChanged is called.
 *
 * @author Jakub Madej
 */
class MyCanvas extends Canvas implements AdjustmentListener {
    private Image image;
    private int scrbarpos;
    private int width;
    private int height;

    /**
     * Creates a new MyCanvas object.
     *
     * @param width width of canvas
     * @param height height of canvas
     */
    MyCanvas(int width, int height) {
        this.width = width;
        this.height = height;
    }

    /**
     * Returns the whole image.
     *
     * @return the whole image
     */
    public Image getImage() {
        return image;
    }

    /**
     * Sets the current posision of visible part of the image.
     *
     * @param pos position
     */
    public void setScrbarPos(int pos) {
        scrbarpos = pos;
    }

    /**
     * Makes this Container displayable by connecting it to a native screen
     * resource.
     */
    public void addNotify() {
        super.addNotify();
        image = createImage(width, ChatPanel.INVISIBLEHEIGHT);

        Graphics g = image.getGraphics();
        g.setColor(ChatPanel.COLOR4);
        g.fillRect(0, 0, width, ChatPanel.INVISIBLEHEIGHT);
        g.setColor(ChatPanel.COLOR3);
        g.drawLine(ChatPanel.LINE1X, 0, ChatPanel.LINE1X,
            ChatPanel.INVISIBLEHEIGHT);
        g.drawLine(ChatPanel.LINE2X, 0, ChatPanel.LINE2X,
            ChatPanel.INVISIBLEHEIGHT);
        scrbarpos = ChatPanel.INVISIBLEHEIGHT - height;
    }

    /**
     * Paints the previously set piece of image.
     *
     * @param g Graphics object to paint on
     */
    public void paint(Graphics g) {
        g.drawImage(image, 0, -scrbarpos, this);
    }

    /**
     * Updates this component.
     *
     * @param g Graphics object to paint on
     */
    public void update(Graphics g) {
        paint(g);
    }

    /**
     * Invoked when the value of the adjustable has changed. Displays proper
     * part of the image.
     *
     * @param e AdjustmentEvent object
     */
    public void adjustmentValueChanged(AdjustmentEvent e) {
        scrbarpos = e.getValue();
        repaint();
    }
}
