package madqba.awt;

import java.awt.Button;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


/**
 * This class is a simple button, on which an Image can be drawn.
 *
 * @author Jakub Madej
 */
public class SmileButton extends Button implements ActionListener {
    Image image;
    SMenu smenu;

    /**
     * Creates a new SmileButton object. The button will be used to open or
     * close passed SMenu.
     *
     * @param image Image object
     * @param smenu SMenu object
     */
    public SmileButton(Image image, SMenu smenu) {
        this.image = image;
        this.smenu = smenu;
        addActionListener(this);
    }

    /**
     * Creates a new SmileButton object.
     *
     * @param image Image object
     */
    SmileButton(Image image) {
        this.image = image;
    }

    /**
     * Updates this component.
     *
     * @param g the specified context to use for updating
     */
    public void update(Graphics g) {
        paint(g);
    }

    /**
     * Paints this component.
     *
     * @param g the graphics context to use for painting
     */
    public void paint(Graphics g) {
        super.paint(g);
        g.drawImage(image, 1, 1, this);
    }

    /**
     * Invoked when an action occurs.
     *
     * @param e ActionEvent object
     */
    public void actionPerformed(ActionEvent e) {
        if (smenu.isVisible())
            smenu.setVisible(false);
        else
            smenu.setVisible(true);
    }
}
