package madqba.awt;

import madqba.awt.chat.Mediator;

import java.awt.TextField;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;


/**
 * This class is an extended textfield designed to handle history, nickname
 * completition and TextCommand objects.
 *
 * @author Jakub Madej
 */
public class TextEntry extends TextField implements KeyListener, FocusListener {
    private String[] msgHistory = new String[10];
    private int hIndex = -1;
    private TextCommand cmd;
    private Mediator med;

    /**
     * Creates a new TextEntry object.
     *
     * @param cmd TextCommand object
     * @param med Mediator object
     */
    public TextEntry(TextCommand cmd, Mediator med) {
        this(cmd);
        this.med = med;
    }

    /**
     * Creates a new TextEntry object.
     *
     * @param TextCommand object
     */
    public TextEntry(TextCommand cmd) {
        this.cmd = cmd;

        for (int i = 0; i < 10; i++)
            msgHistory[i] = "";

        addKeyListener(this);
        addFocusListener(this);
    }

    /**
     * Invoked whenever components looses focus. This method is used to detect
     * if the tab key was pressed and completethe nickname (only if Mediator
     * object was passed to constructor).
     *
     * @param e FocusEvent object
     */
    public void focusLost(FocusEvent e) {
        if (!e.isTemporary())
            requestFocus();
    }

    /**
     * This method isn't used, but it has to be implemented.
     *
     * @param e FocusEvent object
     */
    public void focusGained(FocusEvent e) {
    }

    /**
     * This method isn't used, but it has to be implemented.
     *
     * @param e KeyEvent object
     */
    public void keyTyped(KeyEvent e) {
    }

    /**
     * This method isn't used, but it has to be implemented.
     *
     * @param e KeyEvent object
     */
    public void keyReleased(KeyEvent e) {
    }

    /**
     * This method is invoked whenever a key is pressed. It's used to handle
     * history (arrows) and message sending (enter).
     *
     * @param e KeyEvent object
     */
    public void keyPressed(KeyEvent e) {
        if ((e.getKeyCode() == e.VK_ENTER) && !getText().equals("")) {
            cmd.execute(getText());
            hIndex = -1;

            for (int i = 9; i > 0; i--)
                msgHistory[i] = msgHistory[i - 1];

            msgHistory[0] = getText();
            setText("");
        } else if (e.getKeyCode() == e.VK_DOWN) {
            hIndex--;

            if (hIndex < 0) {
                setText("");
                hIndex = -1;
            } else
                setText(msgHistory[hIndex]);

            setCaretPosition(1000);
        } else if (e.getKeyCode() == e.VK_UP) {
            hIndex++;

            if (hIndex > 9)
                hIndex = 9;

            setText(msgHistory[hIndex] + " ");
            setCaretPosition(1000);
        }
    }
}
