package madqba.net;

import madqba.awt.chat.Mediator;

import org.apache.log4j.Logger;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import java.net.Socket;

import java.util.Date;
import java.util.StringTokenizer;


/**
 * This class is a thread which attempts to connect the client to a server.
 *
 * @author Madej Jakub
 */
public class ConnectProcess extends Thread {
    private static Logger log = Logger.getRootLogger();
    private String serv;
    private String chan;
    private String pass;
    private String nick;
    private int port;
    private Mediator med;

    /**
     * Creates a new ConnectProcess object.
     *
     * @param med Mediator object
     * @param serv server to connect to
     * @param port port to connect to
     * @param chan channel to join
     * @param nick user's nickname
     * @param pass optional bnc password
     */
    public ConnectProcess(Mediator med, String serv, int port, String chan,
        String nick, String pass) {
        this.med = med;
        this.serv = serv;
        this.port = port;
        this.chan = chan;
        this.nick = nick;
        this.pass = pass;
        start();
    }

    /**
     * Main loop which interacts whith server.
     */
    public void run() {
        Socket s = null;
        OutputStreamWriter out;
        BufferedReader in;

        try {
            s = new Socket(serv, port);
            in = new BufferedReader(new InputStreamReader(s.getInputStream()));
            out = new OutputStreamWriter(s.getOutputStream());
            out.write("PASS " + pass + "\r\n");
            out.write("CONN krakow.irc.pl\r\n");
            out.write("PASS fad7f623hf87bz8\r\n");
            out.write("NICK " + nick + "\r\n");
            out.write("USER * 0 * :Java rules\r\n");
            out.flush();
        } catch (Exception ex) {
            ex.printStackTrace();
            med.connError(s);

            return;
        }

        String topic = null;
        long time = (new Date()).getTime();

        for (;;) {
            String msg = null;

            try {
                sleep(1);

                if (in.ready()) {
                    msg = in.readLine();
                    log.info(msg);
                }
            } catch (Exception ex) {
                med.connError(s);

                return;
            }

            if (msg != null) {
                if (msg.equals(
                            "NOTICE AUTH :type /quote help for basic list of commands and usage")) {
                    try {
                        out.write("CONN krakow.irc.pl\r\n");
                        out.flush();
                    } catch (Exception ex) {
                        med.connError(s);

                        return;
                    }
                } else if (msg.equals("NOTICE AUTH :Suceeded connection")) {
                    try {
                        out.write("PASS fad7f623hf87bz8\r\n");
                        out.write("NICK " + nick + "\r\n");
                        out.write("USER * 0 * :Java rules\r\n");
                        out.flush();
                    } catch (Exception ex) {
                        med.connError(s);

                        return;
                    }
                } else if (msg.startsWith("PING :")) {
                    try {
                        out.write("PONG " + msg.substring(6) + "\r\n");
                        out.flush();
                    } catch (Exception ex) {
                        med.connError(s);

                        return;
                    }
                } else if (msg.substring(msg.indexOf(" ") + 1,
                            msg.indexOf(" ", msg.indexOf(" ") + 1)).equals("433")) {
                    med.poniechaj();
                    med.setStatus("Nick " + nick + " " +
                        med.getProperties().getProperty("zajety"));

                    try {
                        s.close();
                    } catch (Exception ex) {
                    }

                    break;
                } else if (msg.substring(msg.indexOf(" ") + 1,
                            msg.indexOf(" ", msg.indexOf(" ") + 1)).equals("001")) {
                    try {
                        out.write("JOIN " + chan + "\r\n");
                        out.flush();
                    } catch (Exception ex) {
                        med.connError(s);

                        return;
                    }
                } else if (msg.substring(msg.indexOf(" ") + 1,
                            msg.indexOf(" ", msg.indexOf(" ") + 1)).equals("332"))
                    topic = msg.substring(msg.indexOf(":", 2) + 1);
                else if (msg.substring(msg.indexOf(" ") + 1,
                            msg.indexOf(" ", msg.indexOf(" ") + 1)).equals("353")) {
                    msg = msg.substring(msg.indexOf(":", 2) + nick.length() +
                            2);
                    med.clearList();

                    StringTokenizer strtok = new StringTokenizer(msg);

                    while (strtok.hasMoreTokens()) {
                        String nicks = strtok.nextToken();

                        if (nicks.startsWith("@") || nicks.startsWith("+"))
                            nicks = nicks.substring(1);

                        if (!nicks.equals(""))
                            med.addToList(nicks);
                    }

                    med.addToList(nick);
                    med.chatMode(topic, out, s);

                    break;
                }
            }

            if ((new Date()).getTime() > (time + (1000 * 60))) {
                med.poniechaj();
                med.setStatus(med.getProperties().getProperty("uplynal"));

                try {
                    s.close();
                } catch (Exception ex) {
                }

                break;
            }
        }
    }
}
